import mimetools, mimetypes, smtplib
import os
from cStringIO import StringIO

from email import Header
from email.Charset import *
from email import Encoders
from email.Message import Message
from email.MIMEAudio import MIMEAudio
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.MIMEText import MIMEText
import email.Utils

class MyObject:
    pass

if not __builtins__.has_key("object"):
    __builtins__["object"] = MyObject

class MailWithAttachment(object):
    def __init__(self, _from, _smtp, _charset):
        self._msg = MIMEMultipart()
        self._msg['From'] = _from
        self._smtp = _smtp
        self._charset = _charset
        self._id = email.Utils.make_msgid()
        self._msg['Message-ID'] = self._id

    def setId(self, _id):
        """
        Set id match string
        """
        self._id = email.Utils.make_msgid(_id)
        self._msg['Message-ID'] = self._id

    def setFrom(self, _from):
        """
        Set the sender address
        """
        self._msg['From'] = _from

    def getFrom(self):
        """
        Return the sender address, or None if not defined
        """
        return self._msg['From']

    def setSmtp(self, _stmp):
        """
        Set the SMTP Server
        """
        self._smtp = _smtp

    def setTo(self, to):
        """
        Set the recipient address
        """
        self._msg['To'] = to

    def getTo(self):
        """
        Return the recipient address, or None if not defined
        """
        if self._msg.hasattr('To'):
            return self._msg['To']
        return None

    def setSubject(self, subject):
        """
        Set the mail subject
        """
        self._msg['Subject'] = str(Header.Header(subject, self._charset))
        self._msg.preambule = self._msg['Subject']
        # Guarantees the message ends in a newline
        self._msg.epilogue = ''

    def getSubject(self):
        """
        Return the mail subject
        """
        return self._msg['Subject']

    def setReply(self, reply):
        """
        Set the reply-to field
        """
        self._msg['Reply-to'] = reply

    def setBody(self, body):
        """
        Try to encode the body text 'iso-8859-1', and add it to the message
        """
        self._text = MIMEText(body, _subtype='plain', _charset=self._charset)
        self._msg.attach(self._text)

    def addAttachment(self, path):
        """
        Add an attachment, trying to guess the mime type
        """
        if not os.path.isfile(path):
            raise IOError, 'InvalidPath %s' % path
        ctype, encoding = mimetypes.guess_type(path)
        if ctype is None or encoding is not None:
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        if maintype == 'text':
            fp = open(path)
            # Note: we should handle calculating the charset
            attach = MIMEText(fp.read(), _subtype=subtype, _charset=self._charset)
            fp.close()
        elif maintype == 'image':
            fp = open(path, 'rb')
            attach = MIMEImage(fp.read(), _subtype=subtype)
            fp.close()
        elif maintype == 'audio':
            fp = open(path, 'rb')
            attach = MIMEAudio(fp.read(), _subtype=subtype)
            fp.close()
        else:
            fp = open(path, 'rb')
            attach = MIMEBase(maintype, subtype)
            attach.set_payload(fp.read())
            fp.close()
            # Encode the payload using Base64
            Encoders.encode_base64(attach)
        # Set the filename parameter
        fileName = path.split('/')[-1]
        fileName = fileName.split('\\')[-1]
        fileName = fileName.split(':')[-1]
        self._filename = fileName
        attach.add_header('Content-Disposition', 'attachment', filename=fileName)
        self._msg.attach(attach)

    def getId(self):
        """
        Return the message id
        """
        return self._id
    
    def sendMail(self):
        """
        Send the mail via SMTP
        """
        if self._msg['To']:
            s = smtplib.SMTP(self._smtp)
            s.sendmail(self._msg['From'], self._msg['To'], self._msg.as_string())
            s.close()
        else:
            raise RuntimeError, 'Mailnode: NoRecipientDefined'

