from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from MailWithAttachment import MailWithAttachment
from Globals import InitializeClass, DTMLFile

from random import choice
import poplib, socket
import re, string

import zLOG

VIEW_PERMISSION = 'View'


manage_addMailnodeForm=DTMLFile('dtml/addMailnode_form', globals())


def manage_addMailnode( self, id, REQUEST=None ):
    """
    Add a Mailnode into the system
    """

    obj = Mailnode(id)
    self._setObject(id, obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

add = manage_addMailnode



class Mailnode(PropertyManager, SimpleItem):
    """
    This tool encapsulates mail send and verification.
    """

    meta_type = "Mailnode"
    id = "Mailnode"
    title = "Mailnode mail sender"
    icon_file = 'mailnode.gif'    
    
    security = ClassSecurityInfo()

    manage_options = PropertyManager.manage_options + SimpleItem.manage_options

    def __init__(self, *args, **kwargs):
        """ Init tool properties """

        letters = ''.join([ x for x in string.letters if ord(x) < 128 ])
        chars = letters + string.digits
        msg_id = ''.join([ choice(chars) for i in range(8) ])


        self.sessions = {}
        
        self._setProperty('from_address', '', 'string')
        self._setProperty('smtp_host', '', 'string')
        self._setProperty('charset', '', 'string')
        self._setProperty('pop_server', '', 'string')
        self._setProperty('pop_port', 110, 'int')
        self._setProperty('pop_username', '', 'string')
        self._setProperty('pop_password', '', 'string')
        self._setProperty('error_callback', '', 'string')
        self._setProperty('message_id_catcher', msg_id, 'string')

    def getFrom_address(self):
        """
        Get from address, either from Mailnode or from portal properties
        """
        return self.getProperty("from_address", None)

    def getSmtp_host(self):
        """
        Get smtp host, either from Mailnode or from Mailhost
        """
        value = self.getProperty("smtp_host", None)
        if value:
            return value

        mailhost = self.getParentNode().MailHost
        return "%s:%s" % (mailhost.smtp_host, mailhost.smtp_port)

    def getCharset(self):
        """
        Get charset, either from Mailnode or from portal_properties
        """
        return self.getProperty("charset", 'utf-8')

    def sendMail(self, arg = []):
        """
        Send a mail, and return mail id
        """
        mail = MailWithAttachment(self.getFrom_address(),
                                  self.getSmtp_host(),
                                  self.getCharset())
        mail.setTo(arg['to'])
        mail.setSubject(arg['subject'])
        mail.setBody(arg['body'])
        mail.setId(self.getProperty('message_id_catcher'))
        if arg.has_key('attachment'):
            for attachment in arg['attachment']:
                mail.addAttachment(attachment)
        mail.sendMail()
        return mail.getId()

    security.declareProtected(VIEW_PERMISSION, 'check')
    def check(self):
        """
        Do the pop3 checking
        """
        M = poplib.POP3(self.getProperty('pop_server'),
			self.getProperty('pop_port'))
        M.user(self.getProperty('pop_username'))
        M.pass_(self.getProperty('pop_password'))

        func = self.getParentNode().restrictedTraverse(self.getProperty('error_callback'))
        
        messages = M.list()[1]
        numBadMessages = 0
        reg = re.compile('Message-ID:.*(<.*%s@%s>).*' %
                            (self.getProperty('message_id_catcher'),
                             socket.getfqdn()))
        for i in messages:
            id, _ = i.split() 
            for j in M.retr(id)[1]:
                for line in j.splitlines():
                    err = reg.match(line)
                    if err:
                        numBadMessages += 1
                        func(err.groups()[0].strip())
            M.dele(id)
        M.quit()
        return '%d/%d' % (numBadMessages, len(messages))

InitializeClass(Mailnode)
